import math

def calculate(table):
    """
    Вычисляет эмпирическое значение φ* по 2x2 таблице сопряжённости.

    Параметры:
      table: Список списков 2x2, где
             table[0][0] — количество «есть эффект» в группе 1,
             table[0][1] — количество «нет эффекта» в группе 1,
             table[1][0] — количество «есть эффект» в группе 2,
             table[1][1] — количество «нет эффекта» в группе 2.

    Возвращает:
      phi_star — рассчитанное значение φ*,
    """
    # Извлекаем данные из таблицы
    group1_ef = table[0][0]
    group1_ne = table[0][1]
    group2_ef = table[1][0]
    group2_ne = table[1][1]

    # Суммарные значения по группам
    total_group1 = group1_ef + group1_ne
    total_group2 = group2_ef + group2_ne
    total = total_group1 + total_group2

    # Суммы по столбцам (категориям)
    total_ef = group1_ef + group2_ef
    total_ne = group1_ne + group2_ne

    # Вычисляем ожидаемые частоты для каждой категории
    expected_group1_ef = total_group1 * total_ef / total
    expected_group1_ne = total_group1 * total_ne / total
    expected_group2_ef = total_group2 * total_ef / total
    expected_group2_ne = total_group2 * total_ne / total

    # Вычисляем вклад по каждому элементу (χ²)
    chi_squared_1 = ((group1_ef - expected_group1_ef) ** 2 / expected_group1_ef) + \
                    ((group1_ne - expected_group1_ne) ** 2 / expected_group1_ne)

    chi_squared_2 = ((group2_ef - expected_group2_ef) ** 2 / expected_group2_ef) + \
                    ((group2_ne - expected_group2_ne) ** 2 / expected_group2_ne)

    # Вычисляем φ*
    return math.sqrt(chi_squared_1 + chi_squared_2)

def validate(table):
    group1_ef = table[0][0]
    group1_ne = table[0][1]
    group2_ef = table[1][0]
    group2_ne = table[1][1]

    total_group1 = group1_ef + group1_ne
    total_group2 = group2_ef + group2_ne

    if min(total_group1,total_group2) >= 5: return True
    if min(total_group1,total_group2) == 2 and max(total_group1,total_group2)>=30: return True
    if min(total_group1,total_group2) == 3 and max(total_group1,total_group2)>=7: return True
    if min(total_group1,total_group2) == 4 and max(total_group1,total_group2)>=5: return True
    return False
